/*
 * audioFunctions.h
 *
 *  Created on: Aug 23, 2021
 *      Author: abhay
 */

#ifndef INC_AUDIOFUNCTIONS_H_
#define INC_AUDIOFUNCTIONS_H_

char WavHeaderBuffer[44];

void Wave_Audio_Header(){
	  /* write chunkID, must be 'RIFF'  ----------------------------------------------*/
	  WavHeaderBuffer[0] = 'R';
	  WavHeaderBuffer[1] = 'I';
	  WavHeaderBuffer[2] = 'F';
	  WavHeaderBuffer[3] = 'F';

	  /* Write the file length ----------------------------------------------------*/
	  /* The sampling time: this value will be be written back at the end of the
	   recording opearation.  Example: 661500 Bytes = 0x000A17FC, byte[7]=0x00, byte[4]=0xFC */
	  WavHeaderBuffer[4] = 0x24; //pWavHeader->AudioLength
	  WavHeaderBuffer[5] = 0xe0;
	  WavHeaderBuffer[6] = 0x2e;
	  WavHeaderBuffer[7] = 0x00;

	  /* Write the file format, must be 'WAVE' ------------------------------------*/
	  WavHeaderBuffer[8]  = 'W';
	  WavHeaderBuffer[9]  = 'A';
	  WavHeaderBuffer[10] = 'V';
	  WavHeaderBuffer[11] = 'E';

	  /* Write the format chunk, must be'fmt ' --------------------------------------*/
	  WavHeaderBuffer[12]  = 'f';
	  WavHeaderBuffer[13]  = 'm';
	  WavHeaderBuffer[14]  = 't';
	  WavHeaderBuffer[15]  = ' ';

	  /* Write the length of the 'fmt' data, must be 0x10 -------------------------*/
	  WavHeaderBuffer[16]  = 0x10;
	  WavHeaderBuffer[17]  = 0x00;
	  WavHeaderBuffer[18]  = 0x00;
	  WavHeaderBuffer[19]  = 0x00;

	  /* Write the audio format, must be 0x01 (PCM) -------------------------------*/
	  WavHeaderBuffer[20]  = 0x01;
	  WavHeaderBuffer[21]  = 0x00;

	  /* Write the number of channels, ie. 0x01 (Mono) ----------------------------*/
	  WavHeaderBuffer[22]  = 0x01;
	  WavHeaderBuffer[23]  = 0x00;

	  /* Write the Sample Rate in Hz ----------------------------------------------*/
	  /* Write Little Endian ie. 8000 = 0x00001F40 => byte[24]=0x40, byte[27]=0x00*/
	  /* Write Little Endian ie. 16000 = 0x00003e80 => byte[24]=0x80, byte[27]=0x00*/
	  WavHeaderBuffer[24]  = 0x80;	//(uint8_t)((pWavHeader->SampleRate & 0xFF));//0x40;
	  WavHeaderBuffer[25]  = 0x3e;	//(uint8_t)((pWavHeader->SampleRate >> 8) & 0xFF);//0x1F;
	  WavHeaderBuffer[26]  = 0x00;	//(uint8_t)((pWavHeader->SampleRate >> 16) & 0xFF);//0x00;
	  WavHeaderBuffer[27]  = 0x00;	//(uint8_t)((pWavHeader->SampleRate >> 24) & 0xFF);//0x00;

	  /* Write the Byte Rate ------------------------------------------------------*/
	  /*
	   * ByteRate = SampleRate * NumChannels * BitsPerSample/8
	   * = 8000 * 1 * 24 /8 = 24000 = 5DC0
	   *= 16000 = bb80
	   */
	  WavHeaderBuffer[28]  = 0x80;	//(uint8_t)((pWavHeader->ByteRate & 0xFF));
	  WavHeaderBuffer[29]  = 0xBB; 	//(uint8_t)((pWavHeader->ByteRate >> 8) & 0xFF);
	  WavHeaderBuffer[30]  = 0x00;	//(uint8_t)((pWavHeader->ByteRate >> 16) & 0xFF);
	  WavHeaderBuffer[31]  = 0x00;	//(uint8_t)((pWavHeader->ByteRate >> 24) & 0xFF);

	  /* Write the block alignment ------------------------------------------------*/
	  /*
	   * == NumChannels * BitsPerSample/8
	   * = 1 * 24/8
	   */
	  WavHeaderBuffer[32]  = 0x03;	//pWavHeader->BlockAlign;
	  WavHeaderBuffer[33]  = 0x00;

	  /* Write the number of bits per sample --------------------------------------*/
	  WavHeaderBuffer[34]  = 0x18;//pWavHeader->BitsPerSample;
	  WavHeaderBuffer[35]  = 0x00;

	  /* Write the Data chunk, must be 'data' ---------------------------------------*/
	  WavHeaderBuffer[36]  = 'd';
	  WavHeaderBuffer[37]  = 'a';
	  WavHeaderBuffer[38]  = 't';
	  WavHeaderBuffer[39]  = 'a';

	  /* Write the number of sample data ------------------------------------------*/
	  /* This field will be overwritten at the end of the recording operation */
	  /*Little endian
	   * num of sample = 1000 * 1024
	   * Channel = 1
	   * Bits per sample = 24
	   * 1000 * 1024 * 24
	   * ----------------        =    sub chunk size
	   *       8
	   */
	  WavHeaderBuffer[40]  = 0x00;
	  WavHeaderBuffer[41]  = 0xe0;
	  WavHeaderBuffer[42]  = 0x2e;
	  WavHeaderBuffer[43]  = 0x00;
}

#endif /* INC_AUDIOFUNCTIONS_H_ */
